package ee.sk.digidoc;

import java.io.Serializable;
import java.security.cert.X509Certificate;

import nu.xom.Element;
import ee.sk.digidoc.utils.XMLObject;

class CounterSignature extends XMLObject implements Serializable{
	
	private SignedDoc m_sigDoc;
	private String m_id;
	private Element m_target;
	private X509Certificate m_cert;
	private Signature m_included_signature=null;
	private String signatureValueId=null;
	
	private static String COUNTER_TYPE=null;
	
	public CounterSignature(Element target, X509Certificate cert,SignedDoc doc) throws DigiDocException {
		m_target = target;
		m_id=target.getAttributeValue("Id")+"-CounterSignature";
		m_cert=cert;
		m_sigDoc=doc;
		m_included_signature=prepareCounterSignature();
		//http://uri.etsi.org/01903#CountersignedSignature
		COUNTER_TYPE=doc.getFormatNamespace()+"#CountersignedSignature";
		createElement();
	}
	
	private Signature prepareCounterSignature() throws DigiDocException{
		if (m_target!=null){
			Signature signature=new Signature(m_sigDoc);
			signature.setId(m_sigDoc.getNewSignatureId());
			SignedInfo si=new SignedInfo(signature);
			si.setCanonicalizationMethod(SignedDoc.CANONICALIZATION_METHOD_20010315);
			si.setSignatureMethod(SignedDoc.RSA_SHA1_SIGNATURE_METHOD);
			
			String svUrl=m_target.getAttributeValue("Id");
			if (svUrl==null){
				signature.getSignatureValue().setId(signature.getId()+"-Sig");
				svUrl=signature.getSignatureValue().getId();
			}
			byte[] canonicalized = SignedDoc.getCanonicalizeInstance().canonicalize(m_target,SignedDoc.CANONICALIZATION_METHOD_20010315);
			byte[] canonicalizedDigest=SignedDoc.digest(canonicalized);
			Reference refsv=new Reference(si,"#"+svUrl,COUNTER_TYPE,SignedDoc.SHA1_DIGEST_ALGORITHM,canonicalizedDigest,new String[]{SignedDoc.CANONICALIZATION_METHOD_20010315});
			si.addReference(refsv);
			
			KeyInfo ki = new KeyInfo(m_cert);
			signature.setKeyInfo(ki);
			ki.setSignature(signature);
			
			CertValue cval = new CertValue();
			cval.setType(CertValue.CERTVAL_TYPE_SIGNER);
			cval.setCert(m_cert);
			signature.addCertValue(cval);
			CertID cid = new CertID(signature, m_cert, CertID.CERTID_TYPE_SIGNER);
			signature.addCertID(cid);
			
			SignedProperties sp = new SignedProperties(signature, m_cert, null, null);
			Reference refsp = new Reference(si, sp);
			refsp.addTransformAlgorithm(SignedDoc.CANONICALIZATION_METHOD_20010315);
			si.addReference(refsp);
			
			signature.setSignedInfo(si);
			signature.setSignedProperties(sp);
			signature.setSignatureValue(new byte[0]);
//			signature.getSignatureValue().setId(svUrl);
			signatureValueId=signature.getSignatureValue().getId();
			return signature;
		}else{
			throw new DigiDocException(DigiDocException.ERR_UNKNOWN,"No signature to countersign.",null);
		}
	}
	
	String getSignatureValueId() {
		return signatureValueId;
	}
	
	public Element getElement() throws DigiDocException {
		return createElement();
	}
	
	protected Element createElement() throws DigiDocException {
		if (thisObject==null)
			thisObject=makeEasyElement("CounterSignature",m_sigDoc.getFormatNamespace());
		if (thisObject.getChildCount()>0)
			thisObject.removeChildren();
		
		Element signature=m_included_signature.createElement();
		if (signature.getParent()!=null)
			signature.detach();
		thisObject.appendChild(signature);
		
		return thisObject;
	}
	
	
	protected Element createElement(String namespace) throws DigiDocException {
		return createElement();
	}
	
	public Signature getSignatureMember() {
		return m_included_signature;
	}
	
	
}
