package ee.sk.digidoc;

import java.io.Serializable;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.Node;
import ee.sk.digidoc.utils.XMLObject;

public class QualifyingProperties extends XMLObject implements Serializable{

	private static final String LOCALNAME = "QualifyingProperties";
	private Signature m_signature;
	private SignedProperties m_signed;
	private UnsignedProperties m_unsigned;
	private String m_target;
		
	
	public QualifyingProperties(){
		m_signature=null;
		m_signed=null;
		m_unsigned=null;
		m_target=null;
	}
	
	public QualifyingProperties(Signature sig){
		m_signature=sig;
		m_target="#"+sig.getId();
		m_signed=null;
		m_unsigned=null;
	}
	
	public Signature getSignature() {
		return m_signature;
	}

	public void setSignature(Signature sig) {
		this.m_signature = sig;
		setParent(m_signature);
		try {
			createElement();
		} catch (DigiDocException e) {
			;
		}
	}

	public SignedProperties getSignedProperties() {
		return m_signed;
	}

	public void setSignedProperties(SignedProperties signed) {
		this.m_signed = signed;
		try {
			createElement();
		} catch (DigiDocException e) {
			;
		}
	}

	public String getTarget() {
		return m_target;
	}

	public void setTarget(String target) {
		this.m_target = target;
		try {
			createElement();
		} catch (DigiDocException e) {
			;
		}
	}

	public UnsignedProperties getUnsignedProperties() {
		return m_unsigned;
	}

	public void setUnsignedProperties(UnsignedProperties unsigned) {
		this.m_unsigned = unsigned;
		try {
			createElement();
		} catch (DigiDocException e) {
			;
		}
	}

	
	
	protected Element createElement() throws DigiDocException{
		String format=m_signature.getSignedDoc().getFormat();
		String namespace=null;
		if (format.equals(SignedDoc.FORMAT_XAdES_1_1_1))
			namespace=XMLNS_ETSI_1_1_1;
		else if (format.equals(SignedDoc.FORMAT_XAdES_1_3_2))
			namespace=XMLNS_ETSI_1_3_2;
		else 
			return null;
		return createElement(namespace);
	}

	protected Element createElement(String namespace) throws DigiDocException {
		if (thisObject==null)
			thisObject=makeEasyElement("QualifyingProperties",namespace);
		
			thisObject.addAttribute(new Attribute("Id",m_signature.getId()+"-QualifyingProperties"));
			thisObject.addAttribute(new Attribute("Target","#"+m_signature.getId()));
		if (thisObject.getChildCount()>0)
			thisObject.removeChildren();
		if (m_signed!=null){
			if(thisObject.getChildElements("SignedProperties",namespace).size()==0){
				Element signedProp=m_signed.createElement();
				if (signedProp.getParent()!=null)
					signedProp.detach();
				thisObject.appendChild(signedProp);
			}else{
				Node n=thisObject.getChildElements("SignedProperties",namespace).get(0);
				thisObject.removeChild(n);
				Element signedProp=m_signed.createElement();
				if (signedProp.getParent()!=null)
					signedProp.detach();
				thisObject.appendChild(signedProp);
			}
		}
		if (m_unsigned!=null){
			if(thisObject.getChildElements("UnsignedProperties",namespace).size()==0){
				Element unsignedProp=m_unsigned.createElement();
				if (unsignedProp.getParent()!=null)
					unsignedProp.detach();
				thisObject.appendChild(unsignedProp);
			}else{
				Node n=thisObject.getChildElements("UnsignedProperties",namespace).get(0);
				thisObject.removeChild(n);
				Element unsignedProp=m_unsigned.createElement();
				if (unsignedProp.getParent()!=null)
					unsignedProp.detach();
				thisObject.appendChild(unsignedProp);
			}
		}
		return thisObject;
	}

//	public static QualifyingProperties parseElement(Node n, Signature sig) throws DigiDocException {
//		Element e=(Element)n;
//		QualifyingProperties qp=null;
//		if (e!=null && e.getLocalName().equals(LOCALNAME)){
//			qp=new QualifyingProperties(sig);
//			
//			Elements signed=e.getChildElements("SignedProperties");
//			if (signed.size()>1)
//				throw new DigiDocException(DigiDocException.ERR_PARSE_XML,"Only one SignatureProperties element by Object allowed.",null);
//			else if (signed.size()==1){
//				SignedProperties sp=SignedProperties.parseElement(signed.get(0),qp);
//				qp.setSignedProperties(sp);
//			}
//			
//			Elements unsigned=e.getChildElements("UnsignedProperties");
//			if (signed.size()>1)
//				throw new DigiDocException(DigiDocException.ERR_PARSE_XML,"Only one UnsignatureProperties element by Object allowed.",null);
//			else if (signed.size()==1){
//				UnsignedProperties usp=UnsignedProperties.parseElement(signed.get(0),qp);
//				qp.setUnsignedProperties(usp);
//			}
//		}else{
//			throw new DigiDocException(DigiDocException.ERR_PARSER_WRONG_ELEMENT,"Null member or wrong localname, expected "+LOCALNAME,null);
//		}
//		return qp;
//	}
	
}
