package ee.sk.digidoc;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

import nu.xom.Attribute;
import nu.xom.Element;
import ee.sk.digidoc.utils.XMLObject;

/*
 * SignaturePolicyIdentitifier.java
 * PROJECT: openXAdES
 * DESCRIPTION:  
 * AUTHOR:  MRC MAP
 *==================================================
 * Copyright (C) AS Sertifitseerimiskeskus
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * GNU Lesser General Public Licence is available at
 * http://www.gnu.org/copyleft/lesser.html
 *==================================================
 */

	public class SignaturePolicyIdentitifier extends XMLObject implements Serializable
	{
		public static int SIGNATURE_POLICY_IMPLIED=0;
		public static int SIGNATURE_POLICY_ID=1;
		
		private int m_policy_mode=0;
				
		/** selected digest algorithm */
		private String m_digestAlgorithm;
		/** digest data */
		private String m_digestValue;
		/** transform algorithm */
		private ArrayList m_transformsAlgorithm;
		/** Policy unique identifier */
		private String m_policyId;
		
		/*
		 * Signature Policy Qualifiers:
		 */
		private String m_spURI;
		
		private String m_spNoticeRef;
		
		private String m_spExplicitText;
		
		public SignaturePolicyIdentitifier() {
			m_digestAlgorithm=null;
			m_digestValue=null;
			m_transformsAlgorithm=null;
			m_policyId=null;
			m_spExplicitText=null;
			m_spNoticeRef=null;
			m_spURI=null;
		}
		
		public SignaturePolicyIdentitifier(String algorithm, String value, String id) {
			this();
			m_digestAlgorithm = algorithm;
			m_digestValue = value;
			m_policyId = id;
		}
		
		public SignaturePolicyIdentitifier(String id) {
			this();
			m_policyId = id;
		}

		public String getDigestAlgorithm() {
			return m_digestAlgorithm;
		}

		public void setDigestAlgorithm(String algorithm) throws DigiDocException {
			if (algorithm==null||algorithm.length()<1)
				throw new DigiDocException(0,"Invalid HashAlgorithm",null);
			m_digestAlgorithm = algorithm;
		}

		public String getDigestValueBase64() {
			return m_digestValue;
		}
		
		public void setDigestValueBase64(String value) throws DigiDocException {
			if (value==null||value.length()<1)
				throw new DigiDocException(0,"Invalid DigestValue",null);
			m_digestValue = value;
		}

		public int getPolicy_mode(){
			return m_policy_mode;
		}

		public void setPolicy_mode(int mode) throws DigiDocException{
			if (mode!=SIGNATURE_POLICY_ID&&mode!=SIGNATURE_POLICY_IMPLIED)
				throw new DigiDocException(0,"Invalid policy mode. Only SIGNATURE_POLICY_ID or SIGNATURE_POLICY_IMPLIED allowed.",null);
			this.m_policy_mode = mode;
		}

		public String getPolicyId() {
			return m_policyId;
		}

		public void setPolicyId(String id) throws DigiDocException {
			if (id==null||id.length()<1)
				throw new DigiDocException(0,"Invalid Policy Identifier",null);
			m_policyId = id;
			createElement();
		}

		public ArrayList getTransformAlgorithms() {
			return m_transformsAlgorithm;
		}

		public void setTransformAlgorithms(ArrayList algorithms) throws DigiDocException {
			m_transformsAlgorithm = algorithms;
			createElement();
		}
		
		public void addTransformAlgorithm(String str) 
		throws DigiDocException
		{
			if (str!=null&&str.length()>0){
				if (!m_transformsAlgorithm.contains(str)){
					m_transformsAlgorithm.add(str);
					createElement();
				}
			}else
				throw new DigiDocException(DigiDocException.ERR_TRANSFORM_ALGORITHM,"Null or size 0 transformAlgorithm error.",null);
		}

		public String getQualifierExplicitText() {
			return m_spExplicitText;
		}

		public void setQualifierExplicitText(String explicitText) throws DigiDocException {
			m_spExplicitText = explicitText;
			createElement();
		}

//		public String getQualifierNoticeRef() {
//			return m_spNoticeRef;
//		}
//
//		public void setQualifierNoticeRef(String noticeRef) {
//			m_spNoticeRef = noticeRef;
//		}

		public String getQualifierURI() {
			return m_spURI;
		}

		public void setQualifierURI(String m_spuri) throws DigiDocException {
			m_spURI = m_spuri;
			createElement();
		}


		protected Element createElement() throws DigiDocException{
			throw new DigiDocException(DigiDocException.ERR_XML_CONVERT,"Cannot create element without specific namespace",null);
		}
		
		protected Element createElement(String namespace) throws DigiDocException{
			if(thisObject==null){
				if (m_policy_mode==SIGNATURE_POLICY_ID) {
					if(m_policyId!= null && m_digestValue != null && m_digestAlgorithm != null) 
						thisObject=makeEasyElement("SignaturePolicyId",namespace);
				}else if (m_policy_mode==SIGNATURE_POLICY_IMPLIED) {
					thisObject=makeEasyElement("SignaturePolicyImplied",namespace);
				}else
					throw new DigiDocException(0,"Invalid Policy mode",null);
			}
			if (m_policy_mode==SIGNATURE_POLICY_ID) {
				if(m_policyId != null) {
					if (thisObject.getChildElements("xd:SigPolicyId").size()>0)
						thisObject.removeChild(thisObject.getChildElements("xd:SigPolicyId").get(0));
					Element sigPolicyId=makeEasyElement("SigPolicyId",namespace);
					sigPolicyId.appendChild(m_policyId);
					thisObject.appendChild(sigPolicyId);
				}
				if (m_transformsAlgorithm != null&&
						m_transformsAlgorithm.size()>0){
					if (thisObject.getChildElements("ds:Transforms").size()>0){
						thisObject.removeChild(thisObject.getChildElements("ds:Transforms").get(0));
					}
					Element transforms=makeEasyElement("Transforms",SignedDoc.XMLNS_XMLDSIG);
					Iterator it=m_transformsAlgorithm.iterator();
					while(it.hasNext()){
						Element transform=makeEasyElement("Transform",SignedDoc.XMLNS_XMLDSIG);
						transform.addAttribute(new Attribute("Algorithm",(String)it.next()));
						transforms.appendChild(transform);
					}
					thisObject.appendChild(transforms);
				}
				if(m_digestAlgorithm != null && m_digestValue!=null) {
					if (thisObject.getChildElements("xd:SigPolicyHash").size()>0)
						thisObject.removeChild(thisObject.getChildElements("xd:SigPolicyHash").get(0));
					Element policyHash=makeEasyElement("SigPolicyHash",namespace);
					Element digestMethod=makeEasyElement("DigestMethod",SignedDoc.XMLNS_XMLDSIG);
					digestMethod.addAttribute(new Attribute("Algorithm",m_digestAlgorithm));
					policyHash.appendChild(digestMethod);
					Element digestValue=makeEasyElement("DigestValue",SignedDoc.XMLNS_XMLDSIG);
					digestValue.appendChild(m_digestValue);
					policyHash.appendChild(digestValue);
					
					thisObject.appendChild(policyHash);
				}
				if(m_spURI != null || m_spExplicitText != null || m_spNoticeRef != null) {
					if (thisObject.getChildElements("xd:SigPolicyQualifiers").size()>0)
						thisObject.removeChild(thisObject.getChildElements("xd:SigPolicyQualifiers").get(0));
					Element policyQualifiers=makeEasyElement("SigPolicyQualifiers",namespace);
					if (m_spURI!=null && m_spURI.length()>0) {
						Element spURI=makeEasyElement("SPURI", namespace);
						spURI.appendChild(m_spURI);
						policyQualifiers.appendChild(spURI);
					}
					//TODO: insertar NoticeRefs
					if (m_spExplicitText!=null && m_spExplicitText.length()>0) {
						Element spUserNotice=makeEasyElement("SPUserNotice", namespace);
						Element spExplicittext=makeEasyElement("spExplicittext", namespace);
						spExplicittext.appendChild(m_spExplicitText);
						spUserNotice.appendChild(spExplicittext);
						policyQualifiers.appendChild(spUserNotice);
					}
					thisObject.appendChild(policyQualifiers);
				}
			}

			return thisObject;
		}
}
