/*
 * UnsignedProperties.java
 * PROJECT: JDigiDoc
 * DESCRIPTION: Digi Doc functions for creating
 *	and reading signed documents. 
 * AUTHOR:  Veiko Sinivee, S|E|B IT Partner Estonia
 *==================================================
 * Copyright (C) AS Sertifitseerimiskeskus
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * GNU Lesser General Public Licence is available at
 * http://www.gnu.org/copyleft/lesser.html
 *==================================================
 *
 * THIS CLASS HAS BEEN MODIFIED:
 * -----------------------------
 * MODIFICATION: Althout Notary can be specified, it is not validated, nor 
 * incorporated in output, so Notary is not supported in this version.
 * AUTHOR: Ministerio de Administraciones Pblicas. Gobierno de Espaa.
 */

package ee.sk.digidoc;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

import nu.xom.Element;
import ee.sk.digidoc.utils.XMLObject;


/**
 * Models the unsigned properties of
 * a signature.
 * @author  Veiko Sinivee
 * @version 1.0
 */
public class UnsignedProperties extends XMLObject implements Serializable
{
    private static final String LOCALNAME = "UnsignedProperties";
	/** signature reference */
    private Signature m_signature;
    private ArrayList m_counterSignatures;
    
    /** 
     * Creates new UsignedProperties 
     * Initializes everything to null
     * @param sig signature reference
     */
    public UnsignedProperties(Signature sig) {
        m_signature = sig;
        m_counterSignatures=new ArrayList();
    }
    
    
    /**
     * Accessor for signature attribute
     * @return value of signature attribute
     */
    public Signature getSignature()
    {
    	return m_signature;
    }
    
    public Element createElement(String namespace) throws DigiDocException{
    	return createElement();
    }
    
    public void addCounterSignature(CounterSignature cs){
    	m_counterSignatures.add(cs);
    }
    
    public Element createElement() throws DigiDocException{
    	String format=m_signature.getSignedDoc().getFormat();
		String namespace=null;
		if (format.equals(SignedDoc.FORMAT_XAdES_1_1_1))
			namespace=SignedDoc.XMLNS_ETSI_1_1_1;
		else if (format.equals(SignedDoc.FORMAT_XAdES_1_2_2))
			namespace=SignedDoc.XMLNS_ETSI_1_2_2;
		else if (format.equals(SignedDoc.FORMAT_XAdES_1_3_2))
			namespace=SignedDoc.XMLNS_ETSI_1_3_2;
    	if(thisObject==null)
    		thisObject=makeEasyElement("UnsignedProperties",namespace);
    	if (thisObject.getChildElements("xd:UnsignedSignatureProperties").size()>0)
    		thisObject.removeChildren();
    	if (m_counterSignatures.size()>0){
    		Element unsignedSignatureProps=makeEasyElement("UnsignedSignatureProperties",namespace);
    		Iterator it=m_counterSignatures.iterator();
    		while(it.hasNext()){
    			CounterSignature cs=(CounterSignature)it.next();
    			Element counter=cs.createElement(namespace);
    			if (counter.getParent()!=null)
    				counter.detach();
    			unsignedSignatureProps.appendChild(counter);
    		}
    		thisObject.appendChild(unsignedSignatureProps);	
    	}
		
		/*TODO:Meter contrafirmas*/
		
		return thisObject;
    }


//	public static UnsignedProperties parseElement(Node n, QualifyingProperties qp) throws DigiDocException {
//		Element e=(Element)n;
//		UnsignedProperties usp=null;
//		if (e!=null && e.getLocalName().equals(LOCALNAME)){
//			usp=new UnsignedProperties(qp.getSignature());
//			e.getChildElements("CounterSignature",e.getNamespaceURI());
//		}else{
//			throw new DigiDocException(DigiDocException.ERR_PARSER_WRONG_ELEMENT,"Null member or wrong localname, expected "+LOCALNAME,null);
//		}
//		return usp;
//	}
    
}
