package ee.sk.digidoc.utils;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import ee.sk.digidoc.DigiDocException;

public class CertUtils {
	
	/**
	 * return certificate owners first name
	 * @return certificate owners first name or null
	 */
	public static String getSubjectFirstName(X509Certificate cert) {
		String name = null;
		String dn = cert.getSubjectDN().getName();
		int idx1 = dn.indexOf("CN=");
		if(idx1 != -1) {
			while(idx1 < dn.length() && dn.charAt(idx1) != ',')
				idx1++;
			idx1++;
			int idx2 = idx1;
			while(idx2 < dn.length() && dn.charAt(idx2) != ',' && dn.charAt(idx2) != '/')
				idx2++;
			name = dn.substring(idx1, idx2);        
		}
		return name;
	}
	
	/**
	 * return certificate owners last name
	 * @return certificate owners last name or null
	 */
	public static String getSubjectLastName(X509Certificate cert) {
		String name = null;
		String dn = cert.getSubjectDN().getName();
		int idx1 = dn.indexOf("CN=");
		if(idx1 != -1) {
			idx1 += 2;
			while(idx1 < dn.length() && !Character.isLetter(dn.charAt(idx1)))
				idx1++;
			int idx2 = idx1;
			while(idx2 < dn.length() && dn.charAt(idx2) != ',' && dn.charAt(idx2) != '/')
				idx2++;
			name = dn.substring(idx1, idx2);            
		}
		return name;
	}
	
	/**
	 * return certificate owners personal code
	 * @return certificate owners personal code or null
	 */
	public static String getSubjectPersonalCode(X509Certificate cert) {
		String code = null;
		String dn = cert.getSubjectDN().getName();
		int idx1 = dn.indexOf("CN=");
		//System.out.println("DN: " + dn);
		if(idx1 != -1) {
			while(idx1 < dn.length() && !Character.isDigit(dn.charAt(idx1)))
				idx1++;
			int idx2 = idx1;
			while(idx2 < dn.length() && Character.isDigit(dn.charAt(idx2)))
				idx2++;
			code = dn.substring(idx1, idx2);            
		}
		//System.out.println("Code: " + code);
		return code;
	}
	
	/**
	 * return CN part of DN
	 * @return CN part of DN or null
	 */
	public static String getCommonName(String dn) {
		String name = null;
		if(dn != null) {
			int idx1 = dn.indexOf("CN=");
			if(idx1 != -1) {
				idx1 += 2;
				while(idx1 < dn.length() && 
						!Character.isLetter(dn.charAt(idx1)))
					idx1++;
				int idx2 = idx1;
				while(idx2 < dn.length() && 
						dn.charAt(idx2) != ',' && 
						dn.charAt(idx2) != '/')
					idx2++;
				name = dn.substring(idx1, idx2);            
			}
		}
		return name;
	}
	
	/**
	 * Reads X509 certificate from a data stream
	 * @param data input data in Base64 form
	 * @return X509Certificate object
	 * @throws EFormException for all errors
	 */
	public static X509Certificate readCertificate(byte[] data)
	throws DigiDocException 
	{
		X509Certificate cert = null;
		try {
			//ByteArrayInputStream certStream = new ByteArrayInputStream(Base64Util.decode(data));
			ByteArrayInputStream certStream = new ByteArrayInputStream(data);
			CertificateFactory cf = CertificateFactory.getInstance("X.509");
			cert = (X509Certificate)cf.generateCertificate(certStream);
			certStream.close();
		} catch(Exception ex) {
			DigiDocException.handleException(ex, DigiDocException.ERR_READ_CERT);
		}
		return cert;        
	}    
}
