package ee.sk.digidoc.utils;

import java.io.UnsupportedEncodingException;
import java.util.Hashtable;

import nu.xom.Element;
import ee.sk.digidoc.DigiDocException;

public abstract class XMLObject{

	protected Element thisObject=null;
	
	protected static String CODEPAGE="UTF-8";
    public static final String XMLNS_XMLDSIG 					= "http://www.w3.org/2000/09/xmldsig#";
    public static final String XMLNS_ETSI_1_1_1 				= "http://uri.etsi.org/01903/v1.1.1#";
//    public static final String XMLNS_ETSI_1_2_2 				= "http://uri.etsi.org/01903/v1.2.2#";
    public static final String XMLNS_ETSI_1_3_2 				= "http://uri.etsi.org/01903/v1.3.2#";
	
    private static Hashtable namespaces=null;
    
    private static final String XADES_PREFIX="xades";
    private static final String DS_PREFIX="ds";
    static{
    	namespaces=new Hashtable();
    	namespaces.put(XMLNS_ETSI_1_1_1,XADES_PREFIX);
//    	namespaces.put(XMLNS_ETSI_1_2_2,XADES_PREFIX);
    	namespaces.put(XMLNS_ETSI_1_3_2,XADES_PREFIX);
    	namespaces.put(XMLNS_XMLDSIG,DS_PREFIX);
    }
	
	final protected byte[] toXML() throws DigiDocException{
		try {
			return thisObject.toXML().getBytes(CODEPAGE);
		} catch (UnsupportedEncodingException e) {
			throw new DigiDocException(DigiDocException.ERR_ENCODING,CODEPAGE+" encoding not supported.",e);
		}
	}
	
	final protected Element makeEasyElement(String tag,String uri) throws DigiDocException{
		String prefix=(String)namespaces.get(uri);
		if (prefix==null){
			throw new DigiDocException(DigiDocException.ERR_DIGIDOC_FORMAT,"Invalid URI: "+uri,null);
		}
    	Element e=new Element(tag,uri);
		e.setNamespacePrefix(prefix);
		e.setNamespaceURI(uri);
		return e;
    }
	
	protected void setParent(XMLObject parent){
		thisObject.detach();
		parent.thisObject.appendChild(thisObject);		
	}
	
	protected void addChild(Element child){
		thisObject.appendChild(child);
	}
	
	public Element getElement() throws DigiDocException{
		createElement();
		return thisObject;
	}
	
	final public String toString() {
        String str = null;
        try {
            str = new String(toXML(),CODEPAGE);
        } catch(Exception ex) {}
        return str;
    }
	
	final public byte[] getXML() throws DigiDocException{
		return this.toXML();
	}
	
	protected abstract Element createElement() throws DigiDocException;
	protected abstract Element createElement(String namespace) throws DigiDocException;
	
//	protected abstract Element createElement(String namespace);
}
